@@ -8,9 +8,12 @@
 
 #include <vdr/plugin.h>
 
+#define MAX_MODES 6
+
 class cMenuSetupAvards : public cMenuSetupPage {
 private:
     cParameter tmpconfig;
+    const char *modes[MAX_MODES];
 protected:
     virtual void Store(void);
 public:
