@@ -1463,7 +1463,7 @@ bool cCam::Active(void)
   cMutexLock lock(this);
   for(cEcmHandler *handler=handlerList.First(); handler; handler=handlerList.Next(handler))
     if(!handler->IsIdle()) {
-      PRINTF(L_GEN_INFO,"Handler %s on card %d is non-idle",handler->Id(),cardNum);
+      PRINTF(L_GEN_INFO,"handler %s on card %d is not idle",handler->Id(),cardNum);
       return true;
       }
   return false;
@@ -1914,7 +1914,7 @@ public:
 
 cScCamSlot::cScCamSlot(cScCiAdapter *ca, int CardIndex, int Slot)
 :cCamSlot(ca)
-,cRingBufferLinear(KILOBYTE(2),5+1)
+,cRingBufferLinear(KILOBYTE(2),5+1,false,"SC-CI slot answer")
 ,checkTimer(-SLOT_CAID_CHECK-1000)
 {
   ciadapter=ca; cardIndex=CardIndex; slot=Slot;
@@ -1931,7 +1931,7 @@ eModuleStatus cScCamSlot::Status(void)
     }
   else if(caids[0]) status=msReady;
   else {
-    status=msNone;
+    status=msPresent; //msNone;
     Check();
     }
   if(status!=lastStatus) {
@@ -2091,7 +2091,7 @@ cScCiAdapter::cScCiAdapter(cDevice *Device, int CardIndex, cCam *Cam)
   memset(version,0,sizeof(version));
   memset(slots,0,sizeof(slots));
   SetDescription("SC-CI adapter on device %d",cardIndex);
-  rb=new cRingBufferLinear(KILOBYTE(5),6+1);
+  rb=new cRingBufferLinear(KILOBYTE(5),6+1,false,"SC-CI adapter read");
   if(rb) {
     rb->SetTimeouts(0,CAM_READ_TIMEOUT);
 /*
@@ -2182,17 +2182,15 @@ void cScCiAdapter::BuildCaids(bool force)
     if((version[0]==0 && c[0]!=0) || memcmp(caids[0],c,sizeof(caids[0]))) {
       memcpy(caids[0],c,sizeof(caids[0]));
       version[0]++;
+      if(version[0]>0) {
+        LBSTART(L_CORE_CI);
+        LBPUT("card %d, slot %d (v=%2d) caids:",cardIndex,0,version[0]);
+        for(int i=0; caids[0][i]; i++) LBPUT(" %04x",caids[0][i]);
+        LBEND();
+        }
       }
     ciMutex.Unlock();
 
-    LBSTART(L_CORE_CI);
-    for(int j=0; j<MAX_CI_SLOTS; j++) {
-      if(version[j]>0) {
-        LBPUT("card %d, slot %d (v=%2d) caids:",cardIndex,j,version[j]); for(int i=0; caids[j][i]; i++) LBPUT(" %04x",caids[j][i]);
-        LBFLUSH();
-        }
-      }
-    LBEND();
     caidTimer.Set(CAID_TIME);
     }
 }
@@ -2421,7 +2419,7 @@ bool cDeCSA::Decrypt(unsigned char *data, int len, bool force)
     LBSTARTF(L_CORE_CSA);
     LBPUT("decrypting %3d packets (ccs=%3d cs=%3d %s)",ccs,ccs,cs,ccs>=cs?"OK ":"INC");
     int n=decrypt_packets(keys[currIdx],range);
-    LBPUT(" -> %3d packets decrypted\n",n);
+    LBPUT(" -> %3d packets decrypted",n);
     if(n>0) return true;
     LBEND();
     }
@@ -2459,7 +2457,7 @@ cDeCsaTSBuffer::cDeCsaTSBuffer(int File, int Size, int CardIndex, cDeCSA *DeCsa,
   f=File; cardIndex=CardIndex; decsa=DeCsa; scActive=ScActive;
   delivered=false;
   lastP=0; lastCount=0;
-  ringBuffer=new cRingBufferLinear(Size,TS_SIZE,true,"TS");
+  ringBuffer=new cRingBufferLinear(Size,TS_SIZE,true,"FFdecsa-TS");
   ringBuffer->SetTimeouts(100,100);
   Start();
 }
@@ -2563,6 +2561,8 @@ void cScDvbDevice::EarlyShutdown(void)
 void cScDvbDevice::LateInit(void)
 {
   int n=CardIndex();
+  if(DeviceNumber()!=n)
+    PRINTF(L_GEN_ERROR,"CardIndex - DeviceNumber mismatch! Put SC plugin first on VDR commandline!");
   if(softcsa) {
     if(HasDecoder()) PRINTF(L_GEN_ERROR,"Card %d is a full-featured card but no ca device found!",n);
     }
@@ -2598,6 +2598,8 @@ void cScDvbDevice::Shutdown(void)
 
 void cScDvbDevice::Startup(void)
 {
+  if(ScSetup.ForceTransfer)
+    SetTransferModeForDolbyDigital(2);
   for(int n=cDevice::NumDevices(); --n>=0;) {
     cScDvbDevice *dev=dynamic_cast<cScDvbDevice *>(cDevice::GetDevice(n));
     if(dev) dev->LateInit();
@@ -2638,6 +2640,10 @@ bool cScDvbDevice::Initialize(void)
     PRINTF(L_GEN_ERROR,"Failed to locate VDR symbols. Plugin not operable");
     return false;
     }
+  if(NumDevices()>0) {
+    PRINTF(L_GEN_ERROR,"Number of devices != 0 on init. Put SC plugin first on VDR commandline! Aborting.");
+    return false;
+    }
   *vdr_nci=0; *vdr_ud=vdr_save_ud;
 
   int i, found=0;
@@ -2736,7 +2742,7 @@ int cScDvbDevice::FindLRUPrg(int source, int transponder, int prg)
 #if APIVERSNUM < 10500
 int cScDvbDevice::ProvidesCa(const cChannel *Channel) const
 {
-  if(Channel->Ca()>=CA_ENCRYPTED_MIN) {
+  if(cam && Channel->Ca()>=CA_ENCRYPTED_MIN) {
     int j;
     caid_t ids[MAXCAIDS+1];
     for(j=0; j<=MAXCAIDS; j++) if((ids[j]=Channel->Ca(j))==0) break;
