@@ -421,12 +421,12 @@ void cNagra2::CreatePQ(const unsigned char *key, BIGNUM *p, BIGNUM *q)
   // expand IDEA-G key
   unsigned char idata[96];
   for(int i=11; i>=0; i--) {
-    const int off=i*8;
-    memcpy(&idata[off],&key[13],8);
-    idata[off]^=i;
-    idea.Decrypt(&idata[off],8,&ks,0);
-    for(int j=7; j>=0; j--) idata[off+j] ^= key[13+j];
-    idata[off]^=i;
+    unsigned char *d=&idata[i*8];
+    memcpy(d,&key[13],8);
+    *d^=i;
+    idea.Decrypt(d,8,&ks,0);
+    xxor(d,8,d,&key[13]);
+    *d^=i;
     }
   // Calculate P
   idata[0] |= 0x80;
@@ -449,7 +449,7 @@ bool cNagra2::Signature(const unsigned char *vkey, const unsigned char *sig, con
     idea.SetEncKey(buff,&ks);
     memcpy(buff,buff+8,8);
     idea.Encrypt(msg+i,8,buff+8,&ks,0);
-    for(int j=7; j>=0; j--) buff[j+8]^=msg[i+j];
+    xxor(&buff[8],8,&buff[8],msg+i);
     }
   buff[8]&=0x7F;
   return (memcmp(sig,buff+8,8)==0);
@@ -617,7 +617,7 @@ bool cSystemNagra2::ProcessECM(const cEcmInfo *ecm, unsigned char *data)
         break;
       case 0x00:
         i+=2; break;
-      case 0x13 ... 0x15:
+      case 0x13 ... 0x17:
         i+=4; break;
       case 0x30 ... 0x36:
       case 0xB0:
@@ -650,6 +650,7 @@ bool cSystemNagra2::ProcessECM(const cEcmInfo *ecm, unsigned char *data)
 
 void cSystemNagra2::ProcessEMM(int pid, int caid, unsigned char *buffer)
 {
+  if (buffer[0]!=0x82) return;
   int cmdLen=buffer[9]-5;
   int id=buffer[10]*256+buffer[11];
 
@@ -660,7 +661,7 @@ void cSystemNagra2::ProcessEMM(int pid, int caid, unsigned char *buffer)
 
   int keyset=(buffer[12]&0x03);
   int sel=(buffer[12]&0x10)<<2;
-  int rsasel=(id==0x4101 || id==0x4001) ? 0:sel; // D+ hack
+  int rsasel=(id==0x4101 || id==0x4001 || id==0x4102) ? 0:sel; // D+ hack
   int sigsel=(buffer[13]&0x80)>>1;
   cPlainKey *pk;
   cBN n;
@@ -734,11 +735,12 @@ void cSystemNagra2::ProcessEMM(int pid, int caid, unsigned char *buffer)
         i+=emmdata[i+1]+2+4;
         break;
       case 0xA6:
-        i+=emmdata[i+1]+1;
+        i+=15;
         break;
-      case 0x13: // Date
-      case 0x14:
-      case 0x15:
+      case 0xAE:
+        i+=11;
+        break;
+      case 0x13 ... 0x17: // Date
         i+=4;
         break;
       case 0xB0 ... 0xBF: // Update with ROM CPU code
