@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.c 1.151 2006/10/07 12:46:22 kls Exp $
+ * $Id: recording.c 1.154 2007/06/17 13:10:12 kls Exp $
  */
 
 #include "recording.h"
@@ -264,6 +264,7 @@ void cResumeFile::Delete(void)
 cRecordingInfo::cRecordingInfo(const cChannel *Channel, const cEvent *Event)
 {
   channelID = Channel ? Channel->GetChannelID() : tChannelID::InvalidID;
+  channelName = Channel ? strdup(Channel->Name()) : NULL;
   ownEvent = Event ? NULL : new cEvent(0);
   event = ownEvent ? ownEvent : Event;
   aux = NULL;
@@ -306,6 +307,7 @@ cRecordingInfo::~cRecordingInfo()
 {
   delete ownEvent;
   free(aux);
+  free(channelName);
 }
 
 void cRecordingInfo::SetData(const char *Title, const char *ShortText, const char *Description)
@@ -336,8 +338,11 @@ bool cRecordingInfo::Read(FILE *f)
            switch (*s) {
              case 'C': {
                          char *p = strchr(t, ' ');
-                         if (p)
+                         if (p) {
+                            free(channelName);
+                            asprintf(&channelName, "%s", compactspace(p));
                             *p = 0; // strips optional channel name
+                            }
                          if (*t)
                             channelID = tChannelID::FromString(t);
                        }
@@ -377,7 +382,7 @@ bool cRecordingInfo::Read(FILE *f)
 bool cRecordingInfo::Write(FILE *f, const char *Prefix) const
 {
   if (channelID.Valid())
-     fprintf(f, "%sC %s\n", Prefix, *channelID.ToString());
+     fprintf(f, "%sC %s%s%s\n", Prefix, *channelID.ToString(), channelName ? " " : "", channelName ? channelName : "");
   event->Dump(f, Prefix, true);
   if (aux)
      fprintf(f, "%s@ %s\n", Prefix, aux);
@@ -408,34 +413,14 @@ char *ExchangeChars(char *s, bool ToFileSystem)
            // The VFAT file system can't handle all characters, so we
            // have to take extra efforts to encode/decode them:
            if (ToFileSystem) {
+              const char *InvalidChars = "\"\\/:*?|<>#";
               switch (*p) {
-                     // characters that can be used "as is":
-                     case '!':
-                     case '@':
-                     case '$':
-                     case '%':
-                     case '&':
-                     case '(':
-                     case ')':
-                     case '+':
-                     case ',':
-                     case '-':
-                     case ';':
-                     case '=':
-                     case '0' ... '9':
-                     case 'a' ... 'z':
-                     case 'A' ... 'Z':
-                     case '': case '':
-                     case '': case '':
-                     case '': case '':
-                     case '':
-                          break;
                      // characters that can be mapped to other characters:
                      case ' ': *p = '_'; break;
                      case '~': *p = '/'; break;
                      // characters that have to be encoded:
                      default:
-                       if (*p != '.' || !*(p + 1) || *(p + 1) == '~') { // Windows can't handle '.' at the end of directory names
+                       if (strchr(InvalidChars, *p) || *p == '.' && (!*(p + 1) || *(p + 1) == '~')) { // Windows can't handle '.' at the end of file/directory names
                           int l = p - s;
                           s = (char *)realloc(s, strlen(s) + 10);
                           p = s + l;
@@ -452,7 +437,7 @@ char *ExchangeChars(char *s, bool ToFileSystem)
                 // mapped characters:
                 case '_': *p = ' '; break;
                 case '/': *p = '~'; break;
-                // encodes characters:
+                // encoded characters:
                 case '#': {
                      if (strlen(p) > 2) {
                         char buf[3];
@@ -502,7 +487,7 @@ cRecording::cRecording(cTimer *Timer, const cEvent *Event)
      Subtitle = " ";
   else if (strlen(Subtitle) > MAX_SUBTITLE_LENGTH) {
      // let's make sure the Subtitle doesn't produce too long a file name:
-     strn0cpy(SubtitleBuffer, Subtitle, MAX_SUBTITLE_LENGTH);
+     Utf8Strn0Cpy(SubtitleBuffer, Subtitle, MAX_SUBTITLE_LENGTH);
      Subtitle = SubtitleBuffer;
      }
   char *macroTITLE   = strstr(Timer->File(), TIMERMACRO_TITLE);
@@ -659,8 +644,8 @@ char *cRecording::StripEpisodeName(char *s)
               s1 = t;
            }
         t++;
-        }
-  if (s1 && s2)
+        } *s1 = 255;
+  if (s1 && s2 && s1 != s && !strchr(".-$", *(s1 - 1)))
      memmove(s1 + 1, s2, t - s2 + 1);
   return s;
 }
@@ -668,7 +653,7 @@ char *cRecording::StripEpisodeName(char *s)
 char *cRecording::SortName(void) const
 {
   if (!sortBuffer) {
-     char *s = StripEpisodeName(strdup(FileName() + strlen(VideoDirectory) + 1));
+     char *s = StripEpisodeName(strdup(FileName() + strlen(VideoDirectory) ));
      strreplace(s, '/', 'a'); // some locales ignore '/' when sorting
      int l = strxfrm(NULL, s, 0) + 1;
      sortBuffer = MALLOC(char, l);
@@ -795,6 +780,15 @@ const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level, bool
   return titleBuffer;
 }
 
+void cRecording::SetStartTime(time_t Start) 
+{
+  start=Start;
+  if (fileName) {
+  	 free(fileName);
+  	 fileName = NULL;
+  	 }
+}
+
 const char *cRecording::PrefixFileName(char Prefix)
 {
   cString p = PrefixVideoFileName(FileName(), Prefix);
