@@ -1555,15 +1555,6 @@ void cTS2PES::send_ipack(void)
             buf[7] = 0x00;
             buf[8] = 0x00;
             count = 9;
-            if (!repacker && subStreamId) {
-               buf[6] = 0x80;
-               buf[7] = 0x01; // pes_extension_flag == 1
-               buf[8] = 0x03; // pes_header_data_length == 3
-               buf[9] = 0x01; // pes_extension_flag_2=1
-               buf[10]= 0x81; // marker_bit=1, pes_extension_data_length=1
-               buf[11] = subStreamId;
-               count = 12;
-               }
             break;
     case 1:
             buf[6] = 0x0F;
@@ -1726,20 +1717,12 @@ void cTS2PES::instant_repack(const uint8_t *Buf, int Count)
           case 7:
                   if (!done && (mpeg == 2 || mpeg1_required > 7)) {
                      flag2 = Buf[c++];
-                     if (!repacker && subStreamId && (flag2 & 0x01)) { // PES_EXTENSION=0x01
-                        esyslog("Error: Cannot add extension to pes packet.");
-                        subStreamId = 0;
-                        }
-                     else
-                        flag2 |= 0x01; // PES_EXTENSION=0x01
                      found++;
                      }
                   break;
           case 8:
                   if (!done && (mpeg == 2 || mpeg1_required > 7)) {
                      hlength = Buf[c++];
-                     if (!repacker && subStreamId)
-                        hlength += 3;
                      found++;
                      if (mpeg == 1 && hlength != 0x0F && (hlength & 0xF0) != 0x20 && (hlength & 0xF0) != 0x30)
                         found = 0; // invalid MPEG1 header
@@ -1783,21 +1766,6 @@ void cTS2PES::instant_repack(const uint8_t *Buf, int Count)
                   return;
                }
 
-            if (!repacker && subStreamId) {
-               // write header one byte at a time
-               // remove from hlength size of our header (3)
-               while (c < Count && found < (hlength + 9 - 3) && found < plength + 6) {
-                     write_ipack(Buf + c, 1);
-                     c++;
-                     found++;
-                     }
-               if (found == (hlength + 9 - 3)) {
-                  uint8_t extheadr[] = {0x01, 0x81};
-                  write_ipack(extheadr, 2);
-                  write_ipack(&subStreamId, 1);
-                  }
-               }
-
             while (c < Count && found < plength + 6) {
                   int l = Count - c;
                   if (l + found > plength + 6)
@@ -1920,11 +1888,13 @@ cRemux::cRemux(int VPid, const int *APids, const int *DPids, const int *SPids, b
      while (*DPids && numTracks < MAXTRACKS && n < MAXDPIDS)
            ts2pes[numTracks++] = new cTS2PES(*DPids++, resultBuffer, IPACKS, 0x00, 0x80 + n++, new cDolbyRepacker);
      }
+  /* future...
   if (SPids) {
      int n = 0;
      while (*SPids && numTracks < MAXTRACKS && n < MAXSPIDS)
            ts2pes[numTracks++] = new cTS2PES(*SPids++, resultBuffer, IPACKS, 0x00, 0x28 + n++);
      }
+  */
 }
 
 cRemux::~cRemux()
